#---
# Excerpted from "Functional Programming: A PragPub Anthology",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/ppanth for more book information.
#---
defmodule CheckDigit do

  import Enum, only: [ reverse: 1, map: 2 ]

  @doc """
  Determine if a sequence of digits is valid, assuming the last digit is
  a Luhn checksum. (http://en.wikipedia.org/wiki/Luhn_algorithm)
  """

  def valid?(numbers) when is_list(numbers) do
    numbers
      |> reverse 
      |> map(&(&1 - ?0))
      |> sum
      |> rem(10) == 0
  end

  defp sum(digits), do: _sum(digits, 0)

  defp _sum([], sum), do: sum
  defp _sum([odd], sum), do: sum + odd
  defp _sum([odd, even | tail], sum) when even < 5 do
    _sum(tail, sum + odd + even*2)
  end
  defp _sum([odd, even | tail], sum) do
    _sum(tail, sum + odd + even*2 - 9)
  end

end

