#---
# Excerpted from "Functional Programming: A PragPub Anthology",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material,
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose.
# Visit http://www.pragmaticprogrammer.com/titles/ppanth for more book information.
#---
defmodule Sequence do
  def fib(0), do: 0
  def fib(1), do: 1
  def fib(n), do: fib(n-1) + fib(n-2)
end
defmodule Parallel do 

  import Enum, only: [map: 2]
  
  # Parallel map
  def pmap(collection, fun) do
    collection |> spawn_children(fun) |> collect_results
  end

  defp spawn_children(collection, fun), do: collection |> map(&spawn_child(&1, fun))

  def spawn_child(item, fun),   do: spawn(__MODULE__, :child, [item, fun, self()])

  def child(item, fun, parent), do: send parent, { self(), fun.(item) }

  defp collect_results(pids),   do: pids |> map(&collect_result_for_pid/1)

  defp collect_result_for_pid(pid) do
     receive do
      { ^pid, value } -> value
    end
  end
end
ExUnit.start

defmodule MyTest do
  use ExUnit.Case

  import Parallel
  import Sequence

  test "basic fib works" do
    assert fib(10) == 55
    assert fib(30) == 832040
  end

  test "parallel map works" do
    assert pmap([1,2,3,4,5,6], &fib/1) == [1, 1, 2, 3, 5, 8]
  end
end

